package FreeBSD::Ports::DistilatorCommon;

use strict;
use warnings;

use File::Basename;
use URI;
use HTTP::Request;
use LWP::UserAgent;

our (@ISA, @EXPORT_OK, @EXPORT);
BEGIN {
        require Exporter;
        @ISA = qw(Exporter);
        @EXPORT = qw(get_urls get_random_user_agent printResult version check_link);
}

# flush after every write
$|=1;

# fake a random user agent
sub get_random_user_agent() {
	my @agents = (
		'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)',
		'Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.4b) Gecko/20030516 Mozilla Firebird/0.6',
		'Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/85 (KHTML, like Gecko) Safari/85',
		'Mozilla/5.0 (Macintosh; U; PPC Mac OS X Mach-O; en-US; rv:1.4a) Gecko/20030401',
		'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.4) Gecko/20030624',
		'Mozilla/5.0 (compatible; Konqueror/3; Linux)',
		'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; de) Opera 8.54',
		'Mozilla/5.0 (X11; U; Linux x86_64; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3',
		'Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.33 Safari/534.3',
		'Mozilla/5.0 Galeon/1.2.8 (X11; Linux i686; U;) Gecko/20030317',
		'Mozilla/5.0 Galeon/1.2.5 (X11; Linux i586; U;) Gecko/20020623 Debian/1.2.5-0.woody.1'
	);

	return $agents[int(rand(scalar(@agents)))];
}

sub check_link($$) {
	my($class,$link)=@_;
	my $ua = LWP::UserAgent->new();
	$ua->env_proxy;
	$ua->timeout($class->{'link_check_timeout'});
	$ua->agent(get_random_user_agent());

	# validate HTTP::Request
	my $request = HTTP::Request->new('HEAD', $link);
	if(!ref $request) {
		return;
	}

	# send HEAD request
	my $response = $ua->request($request);

	# try GET request if return code is >= 400
	if(ref $response and $response->code >= 400) {
		$request = HTTP::Request->new('GET', $link);

		# validate HTTP::Request
		if(!ref $request) {
			return;
		}

		# send GET request but limit response to 512 bytes
		$ua->max_size(512);
		$response = $ua->request($request);
	}

	# something went wrong with the second GET request
	if(!ref $response) {
		return;
	}

	# return final status code
	return $response->code;
}

sub get_urls($$) {
	my($class,$port)=@_;
	my %urls;
	my $make_cmd=sprintf("make -C %s fetch-urlall-list DISTDIR=%s MASTER_SITE_BACKUP='' 2> /dev/null", $port, $class->{'distdir'});
	open(MAKE, $make_cmd.'|');
	while(my $url=<MAKE>) {
		chomp($url);
		if(!exists($urls{$url})) {
			# pre validation to cut off pre-fetch option stuff from some ports
			if($url=~m'((http|https|ftp)://.*)') {
				$url=$1;
				my $uri=URI->new($url);
				if(defined($uri->scheme)) {
					$urls{$url}='1';
				} else {
					printf("WARNING: crap URL (DISTFILE) found in port %s: %s\n", $port, $url);
				}
			}
		}
	}

	# get WWW
	my $pkg_descr=sprintf("%s/pkg-descr", $port);
	if(-f $pkg_descr) {
		open(F,'<'.$pkg_descr);
		while(my $line=<F>) {
			chomp($line);
			if($line=~m'^WWW:\s?((http|https|ftp)://(\S+))') {
				my $url=$1;
				my $uri=URI->new($url);
				if(defined($uri->scheme)) {
					$urls{$url}='2';
				} else {
					printf("WARNING: crap URL (WWW) found in port %s: %s\n", $port, $url);
				}
				
				$urls{$url}='2';
			}
		}
		close(F);
	}

	close(MAKE);
	return \%urls;
}

sub printResult($) {
	my $res=shift;
	my %t=('1' => 'DISTFILE', '2' => 'WWW');

	if($res->{'has_timed_out'}) {
		printf("%s [%s]\t%s (timeout)\n", $res->{'code'}, $t{$res->{'type'}}, $res->{'link'});
	} else {
		printf("%s [%s]\t%s\n", $res->{'code'}, $t{$res->{'type'}}, $res->{'link'});
	}
}

sub version {
        my($class)=@_;
	my $rc=sprintf("%s version %s", basename($0), $class->{'VERSION'});
	return $rc;
}

1;
